@@ -727,180 +727,6 @@ UNSAFE_ENTRY(jclass, Unsafe_DefineClass0(JNIEnv *env, jobject unsafe, jstring na
 } UNSAFE_END
 
 
-// define a class but do not make it known to the class loader or system dictionary
-// - host_class:  supplies context for linkage, access control, protection domain, and class loader
-//                if host_class is itself anonymous then it is replaced with its host class.
-// - data:  bytes of a class file, a raw memory address (length gives the number of bytes)
-// - cp_patches:  where non-null entries exist, they replace corresponding CP entries in data
-
-// When you load an anonymous class U, it works as if you changed its name just before loading,
-// to a name that you will never use again.  Since the name is lost, no other class can directly
-// link to any member of U.  Just after U is loaded, the only way to use it is reflectively,
-// through java.lang.Class methods like Class.newInstance.
-
-// The package of an anonymous class must either match its host's class's package or be in the
-// unnamed package.  If it is in the unnamed package then it will be put in its host class's
-// package.
-//
-
-// Access checks for linkage sites within U continue to follow the same rules as for named classes.
-// An anonymous class also has special privileges to access any member of its host class.
-// This is the main reason why this loading operation is unsafe.  The purpose of this is to
-// allow language implementations to simulate "open classes"; a host class in effect gets
-// new code when an anonymous class is loaded alongside it.  A less convenient but more
-// standard way to do this is with reflection, which can also be set to ignore access
-// restrictions.
-
-// Access into an anonymous class is possible only through reflection.  Therefore, there
-// are no special access rules for calling into an anonymous class.  The relaxed access
-// rule for the host class is applied in the opposite direction:  A host class reflectively
-// access one of its anonymous classes.
-
-// If you load the same bytecodes twice, you get two different classes.  You can reload
-// the same bytecodes with or without varying CP patches.
-
-// By using the CP patching array, you can have a new anonymous class U2 refer to an older one U1.
-// The bytecodes for U2 should refer to U1 by a symbolic name (doesn't matter what the name is).
-// The CONSTANT_Class entry for that name can be patched to refer directly to U1.
-
-// This allows, for example, U2 to use U1 as a superclass or super-interface, or as
-// an outer class (so that U2 is an anonymous inner class of anonymous U1).
-// It is not possible for a named class, or an older anonymous class, to refer by
-// name (via its CP) to a newer anonymous class.
-
-// CP patching may also be used to modify (i.e., hack) the names of methods, classes,
-// or type descriptors used in the loaded anonymous class.
-
-// Finally, CP patching may be used to introduce "live" objects into the constant pool,
-// instead of "dead" strings.  A compiled statement like println((Object)"hello") can
-// be changed to println(greeting), where greeting is an arbitrary object created before
-// the anonymous class is loaded.  This is useful in dynamic languages, in which
-// various kinds of metaobjects must be introduced as constants into bytecode.
-// Note the cast (Object), which tells the verifier to expect an arbitrary object,
-// not just a literal string.  For such ldc instructions, the verifier uses the
-// type Object instead of String, if the loaded constant is not in fact a String.
-
-static InstanceKlass*
-Unsafe_DefineAnonymousClass_impl(JNIEnv *env,
-                                 jclass host_class, jbyteArray data, jobjectArray cp_patches_jh,
-                                 u1** temp_alloc,
-                                 TRAPS) {
-  assert(host_class != NULL, "host_class must not be NULL");
-  assert(data != NULL, "data must not be NULL");
-
-  if (UsePerfData) {
-    ClassLoader::unsafe_defineClassCallCounter()->inc();
-  }
-
-  jint length = typeArrayOop(JNIHandles::resolve_non_null(data))->length();
-  assert(length >= 0, "class_bytes_length must not be negative: %d", length);
-
-  int class_bytes_length = (int) length;
-
-  u1* class_bytes = NEW_C_HEAP_ARRAY_RETURN_NULL(u1, length, mtInternal);
-  if (class_bytes == NULL) {
-    THROW_0(vmSymbols::java_lang_OutOfMemoryError());
-  }
-
-  // caller responsible to free it:
-  *temp_alloc = class_bytes;
-
-  ArrayAccess<>::arraycopy_to_native(arrayOop(JNIHandles::resolve_non_null(data)), typeArrayOopDesc::element_offset<jbyte>(0),
-                                     reinterpret_cast<jbyte*>(class_bytes), length);
-
-  objArrayHandle cp_patches_h;
-  if (cp_patches_jh != NULL) {
-    oop p = JNIHandles::resolve_non_null(cp_patches_jh);
-    assert(p->is_objArray(), "cp_patches must be an object[]");
-    cp_patches_h = objArrayHandle(THREAD, (objArrayOop)p);
-  }
-
-  const Klass* host_klass = java_lang_Class::as_Klass(JNIHandles::resolve_non_null(host_class));
-
-  // Make sure it's the real host class, not another anonymous class.
-  while (host_klass != NULL && host_klass->is_instance_klass() &&
-         InstanceKlass::cast(host_klass)->is_unsafe_anonymous()) {
-    host_klass = InstanceKlass::cast(host_klass)->unsafe_anonymous_host();
-  }
-
-  // Primitive types have NULL Klass* fields in their java.lang.Class instances.
-  if (host_klass == NULL) {
-    THROW_MSG_0(vmSymbols::java_lang_IllegalArgumentException(), "Host class is null");
-  }
-
-  assert(host_klass->is_instance_klass(), "Host class must be an instance class");
-
-  const char* host_source = host_klass->external_name();
-  Handle      host_loader(THREAD, host_klass->class_loader());
-  Handle      host_domain(THREAD, host_klass->protection_domain());
-
-  GrowableArray<Handle>* cp_patches = NULL;
-
-  if (cp_patches_h.not_null()) {
-    int alen = cp_patches_h->length();
-
-    for (int i = alen-1; i >= 0; i--) {
-      oop p = cp_patches_h->obj_at(i);
-      if (p != NULL) {
-        Handle patch(THREAD, p);
-
-        if (cp_patches == NULL) {
-          cp_patches = new GrowableArray<Handle>(i+1, i+1, Handle());
-        }
-
-        cp_patches->at_put(i, patch);
-      }
-    }
-  }
-
-  ClassFileStream st(class_bytes, class_bytes_length, host_source, ClassFileStream::verify);
-
-  Symbol* no_class_name = NULL;
-  ClassLoadInfo cl_info(host_domain,
-                        InstanceKlass::cast(host_klass),
-                        cp_patches,
-                        NULL,     // dynamic_nest_host
-                        Handle(), // classData
-                        false,    // is_hidden
-                        false,    // is_strong_hidden
-                        true);    // can_access_vm_annotations
-
-  InstanceKlass* anonk = SystemDictionary::resolve_from_stream(&st, no_class_name,
-                                                               host_loader,
-                                                               cl_info,
-                                                               CHECK_NULL);
-  assert(anonk != NULL, "no klass created");
-  return anonk;
-}
-
-UNSAFE_ENTRY(jclass, Unsafe_DefineAnonymousClass0(JNIEnv *env, jobject unsafe, jclass host_class, jbyteArray data, jobjectArray cp_patches_jh)) {
-  ResourceMark rm(THREAD);
-
-  jobject res_jh = NULL;
-  u1* temp_alloc = NULL;
-
-  InstanceKlass* anon_klass = Unsafe_DefineAnonymousClass_impl(env, host_class, data, cp_patches_jh, &temp_alloc, THREAD);
-  if (anon_klass != NULL) {
-    res_jh = JNIHandles::make_local(THREAD, anon_klass->java_mirror());
-  }
-
-  // try/finally clause:
-  FREE_C_HEAP_ARRAY(u1, temp_alloc);
-
-  // The anonymous class loader data has been artificially been kept alive to
-  // this point.   The mirror and any instances of this class have to keep
-  // it alive afterwards.
-  if (anon_klass != NULL) {
-    anon_klass->class_loader_data()->dec_keep_alive();
-  }
-
-  // let caller initialize it as needed...
-
-  return (jclass) res_jh;
-} UNSAFE_END
-
-
-
 UNSAFE_ENTRY(void, Unsafe_ThrowException(JNIEnv *env, jobject unsafe, jthrowable thr)) {
   ThreadToNativeFromVM ttnfv(thread);
   env->Throw(thr);
@@ -1003,21 +829,22 @@ UNSAFE_ENTRY(void, Unsafe_Park(JNIEnv *env, jobject unsafe, jboolean isAbsolute,
 
 UNSAFE_ENTRY(void, Unsafe_Unpark(JNIEnv *env, jobject unsafe, jobject jthread)) {
   if (jthread != NULL) {
-    ThreadsListHandle tlh;
-    JavaThread* thr = NULL;
-    oop java_thread = NULL;
-    (void) tlh.cv_internal_thread_to_JavaThread(jthread, &thr, &java_thread);
-    if (java_thread != NULL) {
-      // This is a valid oop.
-      if (thr != NULL) {
-        // The JavaThread is alive.
-        Parker* p = thr->parker();
-        HOTSPOT_THREAD_UNPARK((uintptr_t) p);
-        p->unpark();
-      }
+    oop thread_oop = JNIHandles::resolve_non_null(jthread);
+    // Get the JavaThread* stored in the java.lang.Thread object _before_
+    // the embedded ThreadsListHandle is constructed so we know if the
+    // early life stage of the JavaThread* is protected. We use acquire
+    // here to ensure that if we see a non-nullptr value, then we also
+    // see the main ThreadsList updates from the JavaThread* being added.
+    FastThreadsListHandle ftlh(thread_oop, java_lang_Thread::thread_acquire(thread_oop));
+    JavaThread* thr = ftlh.protected_java_thread();
+    if (thr != nullptr) {
+      // The still live JavaThread* is protected by the FastThreadsListHandle
+      // so it is safe to access.
+      Parker* p = thr->parker();
+      HOTSPOT_THREAD_UNPARK((uintptr_t) p);
+      p->unpark();
     }
-  } // ThreadsListHandle is destroyed here.
-
+  } // FastThreadsListHandle is destroyed here.
 } UNSAFE_END
 
 UNSAFE_ENTRY(jint, Unsafe_GetLoadAverage0(JNIEnv *env, jobject unsafe, jdoubleArray loadavg, jint nelem)) {
@@ -1120,8 +947,6 @@ static JNINativeMethod jdk_internal_misc_Unsafe_methods[] = {
     {CC "writebackPostSync0", CC "()V",                  FN_PTR(Unsafe_WriteBackPostSync0)},
     {CC "setMemory0",         CC "(" OBJ "JJB)V",        FN_PTR(Unsafe_SetMemory0)},
 
-    {CC "defineAnonymousClass0", CC "(" DAC_Args ")" CLS, FN_PTR(Unsafe_DefineAnonymousClass0)},
-
     {CC "shouldBeInitialized0", CC "(" CLS ")Z",         FN_PTR(Unsafe_ShouldBeInitialized0)},
 
     {CC "loadFence",          CC "()V",                  FN_PTR(Unsafe_LoadFence)},
