@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2016, 2019, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2016, 2022, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -39,10 +39,7 @@ public class SafepointCleanupTest {
         OutputAnalyzer output = new OutputAnalyzer(pb.start());
         output.shouldContain("[safepoint,cleanup]");
         output.shouldContain("safepoint cleanup tasks");
-        output.shouldContain("deflating global idle monitors");
-        output.shouldContain("deflating per-thread idle monitors");
         output.shouldContain("updating inline caches");
-        output.shouldContain("compilation policy safepoint handler");
         output.shouldHaveExitValue(0);
     }
 
@@ -57,17 +54,9 @@ public class SafepointCleanupTest {
                                                                   InnerClass.class.getName());
         analyzeOutputOn(pb);
 
-        pb = ProcessTools.createJavaProcessBuilder("-XX:+TraceSafepointCleanupTime",
-                                                   InnerClass.class.getName());
-        analyzeOutputOn(pb);
-
         pb = ProcessTools.createJavaProcessBuilder("-Xlog:safepoint+cleanup=off",
                                                    InnerClass.class.getName());
         analyzeOutputOff(pb);
-
-        pb = ProcessTools.createJavaProcessBuilder("-XX:-TraceSafepointCleanupTime",
-                                                   InnerClass.class.getName());
-        analyzeOutputOff(pb);
     }
 
     public static class InnerClass {
